from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        uprn = kwargs.get("uprn")
        usrn = kwargs.get("paon")
        check_uprn(uprn)
        check_usrn(usrn)

        day = datetime.now().date().strftime("%d")
        month = datetime.now().date().strftime("%m")
        year = datetime.now().date().strftime("%Y")

        api_url = (
            f"https://my.crawley.gov.uk/appshost/firmstep/self/apps/custompage/waste?language=en&uprn={uprn}"
            f"&usrn={usrn}&day={day}&month={month}&year={year}"
        )
        response = requests.get(api_url)

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        titles = [title.text for title in soup.select(".title")]
        collection_tag = soup.body.find_all(
            "div", {"class": "col-md-6 col-sm-6 col-xs-6"}, string="Next collection"
        )
        bin_index = 0
        for tag in collection_tag:
            for item in tag.next_elements:
                if (
                    str(item).startswith('<div class="date text-right text-grey">')
                    and str(item) != ""
                ):
                    collection_date = datetime.strptime(item.text, "%A %d %B")
                    next_collection = collection_date.replace(year=datetime.now().year)
                    if datetime.now().month == 12 and next_collection.month == 1:
                        next_collection = next_collection + relativedelta(years=1)

                    dict_data = {
                        "type": titles[bin_index].strip(),
                        "collectionDate": next_collection.strftime(date_format),
                    }
                    data["bins"].append(dict_data)
                    bin_index += 1
                    break
        return data



import time

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            collections = []

            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless)
            driver.get(
                "https://www.cheshirewestandchester.gov.uk/residents/waste-and-recycling/your-bin-collection/collection-day"
            )

            time.sleep(5)

            cookie_close_button = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located((By.ID, "ccc-close"))
            )
            cookie_close_button.click()

            find_collection_button = WebDriverWait(driver, timeout=10).until(
                EC.presence_of_element_located(
                    (By.LINK_TEXT, "Find your collection day")
                )
            )
            find_collection_button.click()

            banner_close_button = WebDriverWait(driver, timeout=30).until(
                EC.presence_of_element_located((By.ID, "close-cookie-message"))
            )
            banner_close_button.click()

            time.sleep(5)

            frame = driver.find_element(
                By.XPATH, "/html/body/div[4]/section/div/div[2]/div[2]/div/iframe"
            )
            driver.switch_to.frame(frame)

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.NAME, "postcode_search"))
            )
            inputElement_postcode.send_keys(user_postcode)

            address_box_text = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.ID, "label_Choose_Address"))
            )
            address_box_text.click()
            time.sleep(2)

            address_selection_menu = Select(
                driver.find_element(By.ID, "Choose_Address")
            )
            for idx, addr_option in enumerate(address_selection_menu.options):
                option_name = addr_option.text[0 : len(user_paon)]
                if option_name == user_paon:
                    selected_address = addr_option
                    break
            address_selection_menu.select_by_visible_text(selected_address.text)

            WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.XPATH, '//*[@id="bin-schedule-content"]/div/h3')
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            # Get collections
            bin_cards = soup.find_all("div", {"class": "bin-schedule-content-info"})
            for card in bin_cards:
                bin_name = card.contents[0].text.strip() + " bin"
                bin_date = datetime.strptime(
                    card.contents[1].text.split(":")[1].strip(), "%A, %d %B %Y"
                )
                collections.append((bin_name, bin_date))

            ordered_data = sorted(collections, key=lambda x: x[1])
            for item in ordered_data:
                dict_data = {
                    "type": item[0].capitalize(),
                    "collectionDate": item[1].strftime(date_format),
                }
                data["bins"].append(dict_data)

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise

        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


def get_token(page) -> str:
    """
    Get a __token to include in the form data
        :param page: Page html
        :return: Form __token
    """
    soup = BeautifulSoup(page.text, features="html.parser")
    soup.prettify()
    token = soup.find("input", {"name": "__token"}).get("value")
    return token


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        uprn = kwargs.get("uprn")
        postcode = kwargs.get("postcode")
        check_uprn(uprn)
        check_postcode(postcode)

        values = {
            "__token": get_token(page),
            "page": "492",
            "locale": "en_GB",
            "q9f451fe0ca70775687eeedd1e54b359e55f7c10c_0_0": postcode,
            "q9f451fe0ca70775687eeedd1e54b359e55f7c10c_1_0": uprn,
            "next": "Next",
        }
        headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"}
        requests.packages.urllib3.disable_warnings()
        response = requests.request(
            "POST",
            "https://www.welhat.gov.uk/xfp/form/214",
            headers=headers,
            data=values,
        )

        soup = BeautifulSoup(response.text, features="html.parser")

        rows = soup.find("table").find_all("tr")

        # Form a JSON wrapper
        data = {"bins": []}

        # Loops the Rows
        for row in rows:
            cells = row.find_all("td")
            if cells:
                binType = cells[0].get_text(strip=True)
                collectionDate = datetime.strptime(
                    cells[1].get_text(strip=True), "%A %d %B  %Y"
                ).strftime(date_format)

                # Make each Bin element in the JSON
                dict_data = {
                    "type": binType,
                    "collectionDate": collectionDate,
                }

                # Add data to the main JSON Wrapper
                data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        requests.packages.urllib3.disable_warnings()
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        headers = {
            "Accept": "*/*",
            "Accept-Language": "en-GB,en;q=0.6",
            "Connection": "keep-alive",
            "Referer": "https://www.valeofglamorgan.gov.uk/",
            "Sec-Fetch-Dest": "script",
            "Sec-Fetch-Mode": "no-cors",
            "Sec-Fetch-Site": "same-site",
            "Sec-GPC": "1",
            "sec-ch-ua": '"Not?A_Brand";v="8", "Chromium";v="108", "Brave";v="108"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
        }
        params = {
            "RequestType": "LocalInfo",
            "ms": "ValeOfGlamorgan/AllMaps",
            "group": "Community and Living|Refuse HIDE2",
            "type": "json",
            "callback": "AddressInfoCallback",
            "uid": user_uprn,
            "import": "jQuery35108514154283927682_1673022974838",
            "_": "1673022974840",
        }

        # Get a response from the council
        response = requests.get(
            "https://myvale.valeofglamorgan.gov.uk/getdata.aspx",
            params=params,
            headers=headers,
        ).text

        # Load the JSON and seek out the bin week text, then add it to the calendar URL. Also take the weekly
        # collection type and generate dates for it. Then make a GET request for the calendar
        bin_week = str(
            json.loads(response)["Results"]["Refuse_HIDE2"]["Your_Refuse_round_is"]
        ).replace(" ", "-")
        weekly_collection = str(
            json.loads(response)["Results"]["Refuse_HIDE2"]["Recycling__type"]
        ).capitalize()
        weekly_dates = get_weekday_dates_in_period(
            datetime.now(), days_of_week.get(bin_week.split("-")[0].strip()), amount=48
        )
        schedule_url = f"https://www.valeofglamorgan.gov.uk/en/living/Recycling-and-Waste/collections/Black-Bag-Collections/{bin_week}.aspx"
        response = requests.get(schedule_url, verify=False)

        # BS4 parses the calendar
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        # Some scraper variables
        collections = []

        # Get the calendar table and find the headers
        table = soup.find("table", {"class": "TableStyle_Activities"}).find("tbody")
        table_headers = table.find("tr").find_all("th")
        # For all rows below the header, find all details in th next row
        for tr in soup.find_all("tr")[1:]:
            row = tr.find_all("td")
            # Parse month and year - month needs converting from text to number
            month_and_year = row[0].text.split()
            if month_and_year[0] in list(calendar.month_abbr):
                collection_month = datetime.strptime(month_and_year[0], "%b").month
            elif month_and_year[0] == "Sept":
                collection_month = int(9)
            else:
                collection_month = datetime.strptime(month_and_year[0], "%B").month
            collection_year = datetime.strptime(month_and_year[1], "%Y").year

            # Get the collection dates column, remove anything that's not a number or space and then convert to dates
            for day in remove_alpha_characters(row[1].text.strip()).split():
                try:
                    bin_date = datetime(collection_year, collection_month, int(day))
                    collections.append(
                        (
                            table_headers[1]
                            .text.strip()
                            .replace(" collection date", ""),
                            bin_date,
                        )
                    )
                except Exception as ex:
                    continue

        # Add in weekly dates to the tuple
        for date in weekly_dates:
            collections.append(
                (weekly_collection, datetime.strptime(date, date_format))
            )

        # Order all the data, only including future dates
        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for item in ordered_data:
            collection_date = item[1]
            if collection_date.date() >= datetime.now().date():
                dict_data = {
                    "type": item[0],
                    "collectionDate": collection_date.strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data



import time
import urllib.parse
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


def format_bin_type(bin_colour: str):
    bin_types = {
        "grey": "Garden waste (Grey Bin)",
        "brown": "Paper and card (Brown Bin)",
        "blue": "Bottles and cans (Blue Bin)",
        "green": "General waste (Green Bin)",
    }
    bin_colour = urllib.parse.unquote(bin_colour).split(" ")[0].lower()
    return bin_types[bin_colour]


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            user_agent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36"
            driver = create_webdriver(web_driver, headless, user_agent)
            driver.get("https://myaccount.chorley.gov.uk/wastecollections.aspx")

            # Accept cookies banner
            cookieBanner = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "PrivacyPolicyNotification"))
            )
            cookieClose = cookieBanner.find_element(
                By.CSS_SELECTOR, "span.ui-icon-circle-close"
            )
            cookieClose.click()

            # Populate postcode field
            inputElement_postcode = driver.find_element(
                By.ID,
                "MainContent_addressSearch_txtPostCodeLookup",
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "MainContent_addressSearch_btnFindAddress",
                    )
                )
            )
            findAddress.click()

            time.sleep(1)

            # Wait for the 'Select address' dropdown to appear and select option matching UPRN
            dropdown = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.ID,
                        "MainContent_addressSearch_ddlAddress",
                    )
                )
            )
            # Create a 'Select' for it, then select the matching URPN option
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_value(user_uprn)

            # Wait for the submit button to appear, then click it to get the collection dates
            submit = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "MainContent_btnSearch"))
            )
            submit.click()

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # Get the property details
            property_details = soup.find(
                "table",
                {"class": "WasteCollection"},
            )

            # Get the dates
            for row in property_details.tbody.find_all("tr", recursive=False):
                month_col = row.td
                month = month_col.get_text(strip=True)

                for date_col in month_col.find_next_siblings("td"):
                    day = date_col.p.contents[0].strip()

                    if day == "":
                        continue

                    for bin_type in date_col.find_all("img"):
                        bin_colour = bin_type.get("src").split("/")[-1].split(".")[0]
                        date_object = datetime.strptime(f"{day} {month}", "%d %B %Y")
                        date_formatted = date_object.strftime("%d/%m/%Y")

                        dict_data = {
                            "type": format_bin_type(bin_colour),
                            "collectionDate": date_formatted,
                        }
                        data["bins"].append(dict_data)
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import math
from datetime import *

import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        requests.packages.urllib3.disable_warnings()
        data = {"bins": []}
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        # Get form data
        s = requests.session()
        cookies = {
            "ntc-cookie-policy": "1",
            "SSESS6ec6d5d2d471c0357053d5993a839bce": "qBdR7XhmSMd5_PDBIqG0It2R0Fq67igrejRY-WOcskE",
            "has_js": "1",
        }
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8",
            "Accept-Language": "en-GB,en;q=0.7",
            "Cache-Control": "max-age=0",
            "Connection": "keep-alive",
            "Origin": "https://my.northtyneside.gov.uk",
            "Referer": "https://my.northtyneside.gov.uk/category/81/bin-collection-dates",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-User": "?1",
            "Sec-GPC": "1",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            "sec-ch-ua": '"Not?A_Brand";v="8", "Chromium";v="108", "Brave";v="108"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
        }
        ajax_data = {
            "postcode": user_postcode,
            "form_build_id": "form-BQ47tM0NKADE0s8toYkdSef3QBn6lDM-yBseqIOho80",
            "form_id": "ntc_address_wizard",
            "_triggering_element_name": "op",
            "_triggering_element_value": "Find",
            "ajax_html_ids[]": [
                "ntc-web-my",
                "skip-link",
                "navbar",
                "navbar-collapse",
                "search-block-form",
                "ntc-web-search-input-label",
                "ntc-web-search-input",
                "ui-id-1",
                "ntc-web-main",
                "main-content",
                "block-system-main",
                "web-drupal-content",
                "web-drupal-content-main",
                "node-4024",
                "block-ntc-address-ntc-address-finder",
                "wizard-form-wrapper",
                "ntc-address-wizard",
                "edit-postcode",
                "edit-find",
                "backtotop",
            ],
            "ajax_page_state[theme]": "ntc_bootstrap",
            "ajax_page_state[theme_token]": "LN05JIzI6rocWDiBpDyVeywYveuS4jlxD_N0_hhp2Ko",
            "ajax_page_state[css][0]": "1",
            "ajax_page_state[css][modules/system/system.base.css]": "1",
            "ajax_page_state[css][misc/ui/jquery.ui.core.css]": "1",
            "ajax_page_state[css][misc/ui/jquery.ui.theme.css]": "1",
            "ajax_page_state[css][misc/ui/jquery.ui.menu.css]": "1",
            "ajax_page_state[css][misc/ui/jquery.ui.autocomplete.css]": "1",
            "ajax_page_state[css][sites/all/modules/calendar/css/calendar_multiday.css]": "1",
            "ajax_page_state[css][sites/all/modules/date/date_repeat_field/date_repeat_field.css]": "1",
            "ajax_page_state[css][modules/field/theme/field.css]": "1",
            "ajax_page_state[css][modules/node/node.css]": "1",
            "ajax_page_state[css][sites/all/modules/youtube/css/youtube.css]": "1",
            "ajax_page_state[css][sites/all/modules/views/css/views.css]": "1",
            "ajax_page_state[css][sites/all/modules/back_to_top/css/back_to_top.css]": "1",
            "ajax_page_state[css][sites/all/modules/ckeditor/css/ckeditor.css]": "1",
            "ajax_page_state[css][sites/all/modules/ctools/css/ctools.css]": "1",
            "ajax_page_state[css][sites/all/modules/panels/css/panels.css]": "1",
            "ajax_page_state[css][sites/all/modules/taxonomy_access/taxonomy_access.css]": "1",
            "ajax_page_state[css][sites/all/modules/search_autocomplete/css/themes/minimal.css]": "1",
            "ajax_page_state[css][sites/all/themes/ntc_bootstrap/css/bootstrap.css]": "1",
            "ajax_page_state[css][sites/all/themes/ntc_bootstrap/css/generic.css]": "1",
            "ajax_page_state[css][sites/all/themes/ntc_bootstrap/css/custom.css]": "1",
            "ajax_page_state[css][sites/all/themes/ntc_bootstrap/css/components.css]": "1",
            "ajax_page_state[css][sites/all/themes/ntc_bootstrap/css/modules.css]": "1",
            "ajax_page_state[css][sites/all/themes/ntc_bootstrap/css/fostering.css]": "1",
            "ajax_page_state[css][sites/all/themes/ntc_bootstrap/css/responsive.css]": "1",
            "ajax_page_state[css][sites/all/themes/ntc_bootstrap/css/ie10.css]": "1",
            "ajax_page_state[css][sites/all/themes/ntc_bootstrap/css/ie.css]": "1",
            "ajax_page_state[js][0]": "1",
            "ajax_page_state[js][1]": "1",
            "ajax_page_state[js][sites/all/themes/bootstrap/js/bootstrap.js]": "1",
            "ajax_page_state[js][//ajax.googleapis.com/ajax/libs/jquery/1.10.2/jquery.min.js]": "1",
            "ajax_page_state[js][misc/jquery-extend-3.4.0.js]": "1",
            "ajax_page_state[js][misc/jquery-html-prefilter-3.5.0-backport.js]": "1",
            "ajax_page_state[js][misc/jquery.once.js]": "1",
            "ajax_page_state[js][misc/drupal.js]": "1",
            "ajax_page_state[js][//ajax.googleapis.com/ajax/libs/jqueryui/1.10.2/jquery-ui.min.js]": "1",
            "ajax_page_state[js][sites/all/modules/jquery_update/replace/ui/external/jquery.cookie.js]": "1",
            "ajax_page_state[js][sites/all/modules/jquery_update/replace/misc/jquery.form.min.js]": "1",
            "ajax_page_state[js][misc/ajax.js]": "1",
            "ajax_page_state[js][sites/all/modules/jquery_update/js/jquery_update.js]": "1",
            "ajax_page_state[js][sites/all/modules/back_to_top/js/back_to_top.js]": "1",
            "ajax_page_state[js][sites/all/themes/bootstrap/js/misc/_progress.js]": "1",
            "ajax_page_state[js][sites/all/modules/field_group/field_group.js]": "1",
            "ajax_page_state[js][sites/all/modules/search_autocomplete/js/jquery.autocomplete.js]": "1",
            "ajax_page_state[js][sites/all/themes/ntc_bootstrap/scripts/NTC.jquery.contentMenuScroller.js]": "1",
            "ajax_page_state[js][sites/all/themes/ntc_bootstrap/scripts/NTC.jquery.alertClose.js]": "1",
            "ajax_page_state[js][sites/all/themes/ntc_bootstrap/scripts/NTC.jquery.activeTrail.js]": "1",
            "ajax_page_state[js][sites/all/themes/ntc_bootstrap/scripts/NTC.jquery.expandLinkToDiv.js]": "1",
            "ajax_page_state[js][sites/all/themes/ntc_bootstrap/scripts/NTC.jquery.events.js]": "1",
            "ajax_page_state[js][sites/all/themes/ntc_bootstrap/scripts/cookieconsent.js]": "1",
            "ajax_page_state[js][sites/all/themes/ntc_bootstrap/scripts/google-analytics.js]": "1",
            "ajax_page_state[js][sites/all/themes/ntc_bootstrap/scripts/ios-orientationchange-fix.js]": "1",
            "ajax_page_state[js][sites/all/themes/bootstrap/js/misc/ajax.js]": "1",
            "ajax_page_state[jquery_version]": "1.10",
        }
        uprn_data = {
            "house_number": "0000" + f"{user_uprn}",
            "op": "Use",
            "form_build_id": "form-BQ47tM0NKADE0s8toYkdSef3QBn6lDM-yBseqIOho80",
            "form_id": "ntc_address_wizard",
        }
        collections = []

        response = s.post(
            "https://my.northtyneside.gov.uk/system/ajax",
            # cookies=cookies,
            headers=headers,
            data=ajax_data,
            verify=False,
        )
        response = s.post(
            "https://my.northtyneside.gov.uk/category/81/bin-collection-dates",
            # cookies=cookies,
            headers=headers,
            data=uprn_data,
            verify=False,
        )
        response = s.get(
            "https://my.northtyneside.gov.uk/category/81/bin-collection-dates",
            # cookies=cookies,
            headers=headers,
            data=uprn_data,
            verify=False,
        )

        # Parse form page and get the day of week text
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()
        bin_text = soup.find("section", {"class": "block block-ntc-bins clearfix"})
        regular_text = bin_text.select("p:nth-child(2) > strong")[0].text.strip()
        x = bin_text.select("p:nth-child(4) > strong")
        if len(bin_text.select("p:nth-child(4) > strong")) == 1:
            special_text = bin_text.select("p:nth-child(4) > strong")[0].text.strip()
        else:
            special_text = bin_text.select("p:nth-child(5) > strong")[0].text.strip()

        # Since calendar only shows until end of March 2024, work out how many weeks that is
        weeks_total = math.floor((datetime(2024, 4, 1) - datetime.now()).days / 7)

        # Convert day text to series of dates using previous calculation
        regular_collections = get_weekday_dates_in_period(
            datetime.today(),
            days_of_week.get(regular_text.capitalize()),
            amount=weeks_total,
        )
        special_collections = get_weekday_dates_in_period(
            datetime.today(), days_of_week.get(special_text.capitalize())
        )

        # Differentiate between regular and recycling bins
        for item in regular_collections:
            item_as_date = datetime.strptime(item, date_format)
            # Check if holiday (calendar only has one day that's a holiday, and it's moved to the next day)
            if is_holiday(item_as_date, Region.ENG):
                item_as_date += timedelta(days=1)
            # Use the isoweek number to separate collections - at the time of writing 11th Jan is week 2, which
            # is for the grey bin
            if (item_as_date.date().isocalendar()[1] % 2) == 0:
                collections.append(("Regular bin (green)", item_as_date))

            else:
                collections.append(("Recycling bin (grey)", item_as_date))

        # Add the special collection dates to the collection tuple
        collections += [
            ("Special collection (bookable)", datetime.strptime(item, date_format))
            for item in special_collections
        ]

        # Sort the collections tuple by date, the add to dictionary and return
        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



import time

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        data = {"bins": []}

        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        headless = kwargs.get("headless")
        web_driver = kwargs.get("web_driver")
        driver = create_webdriver(web_driver, headless)
        page = "https://www1.arun.gov.uk/when-are-my-bins-collected/"
        check_paon(user_paon)
        check_postcode(user_postcode)
        driver.get(page)

        start_now_button = WebDriverWait(driver, timeout=15).until(
            EC.presence_of_element_located((By.LINK_TEXT, "Start now"))
        )
        start_now_button.click()

        # Wait for the postcode field to appear then populate it
        input_element_postcode = WebDriverWait(driver, 30).until(
            EC.presence_of_element_located((By.ID, "postcode"))
        )
        input_element_postcode.send_keys(user_postcode)

        continue_button = WebDriverWait(driver, timeout=15).until(
            EC.presence_of_element_located((By.CLASS_NAME, "govuk-button"))
        )
        continue_button.click()

        address_selection_menu = Select(driver.find_element(By.ID, "address"))
        for idx, addr_option in enumerate(address_selection_menu.options):
            option_name = addr_option.text[0 : len(user_paon)]
            if option_name == user_paon:
                selected_address = addr_option
                break
        address_selection_menu.select_by_visible_text(selected_address.text)

        continue_button = WebDriverWait(driver, timeout=15).until(
            EC.presence_of_element_located((By.CLASS_NAME, "govuk-button"))
        )
        continue_button.click()
        # Check for text saying "Next collection dates"
        WebDriverWait(driver, 30).until(
            EC.presence_of_element_located(
                (By.XPATH, "//*[contains(text(), 'Next collection dates')]")
            )
        )

        soup = BeautifulSoup(driver.page_source, "html.parser")
        soup.prettify()
        table = soup.find("table", class_="govuk-table")

        for row in table.find("tbody").find_all("tr"):
            # Extract the type of collection and the date of next collection
            collection_type = (
                row.find("th", class_="govuk-table__header").text.strip().split(" ")
            )[0]
            collection_date = row.find("td", class_="govuk-table__cell").text.strip()

            # Append the information to the data structure
            data["bins"].append(
                {"type": collection_type, "collectionDate": collection_date}
            )

        return data



import re
import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"}

            uprn = kwargs.get("uprn")
            postcode = kwargs.get("postcode")
            user_paon = kwargs.get("paon")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            driver = create_webdriver(web_driver, headless)
            url = kwargs.get("url")

            driver.execute_script(f"window.location.href='{url}'")

            wait = WebDriverWait(driver, 120)
            post_code_search = wait.until(
                EC.presence_of_element_located((By.XPATH, '//input[@name="keyword"]'))
            )

            post_code_search.send_keys(postcode)

            submit_btn = wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "__submitButton"))
            )

            submit_btn.send_keys(Keys.ENTER)

            address_results = wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "directories-table"))
            )
            address_link = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f"//a[contains(text(), '{user_paon}')]")
                )
            )

            address_link.send_keys(Keys.ENTER)
            results = wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "usercontent"))
            )

            # Make a BS4 object
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            # Get collection calendar
            calendar_urls = soup.find_all(
                "a", string=re.compile(r"view or download the collection calendar")
            )
            if len(calendar_urls) > 0:
                requests.packages.urllib3.disable_warnings()
                response = requests.get(calendar_urls[0].get("href"), headers=headers)

                # Make a BS4 object
                soup = BeautifulSoup(response.text, features="html.parser")
                soup.prettify()

                # Loop the months
                for month in soup.find_all("div", {"class": "usercontent"}):
                    year = ""
                    if month.find("h2") and "calendar" not in month.find("h2").get_text(
                        strip=True
                    ):
                        year = datetime.strptime(
                            month.find("h2").get_text(strip=True), "%B %Y"
                        ).strftime("%Y")
                    elif month.find("h3"):
                        year = datetime.strptime(
                            month.find("h3").get_text(strip=True), "%B %Y"
                        ).strftime("%Y")
                    if year != "":
                        for row in month.find_all("li"):
                            results = re.search(
                                "([A-Za-z]+ \\d\\d? [A-Za-z]+): (.+)",
                                row.get_text(strip=True),
                            )
                            if results:
                                dict_data = {
                                    "type": results.groups()[1].capitalize(),
                                    "collectionDate": datetime.strptime(
                                        results.groups()[0] + " " + year, "%A %d %B %Y"
                                    ).strftime(date_format),
                                }
                                data["bins"].append(dict_data)

                # Sort collections
                data["bins"].sort(
                    key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
                )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://selfservice.broxtowe.gov.uk/renderform.aspx?t=217&k=9D2EF214E144EE796430597FB475C3892C43C528"

            data = {"bins": []}

            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless)
            driver.get(page)

            # Populate postcode field
            inputElement_postcode = driver.find_element(
                By.ID,
                "ctl00_ContentPlaceHolder1_FF5683TB",
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            driver.find_element(
                By.ID,
                "ctl00_ContentPlaceHolder1_FF5683BTN",
            ).click()

            # Wait for the 'Select address' dropdown to appear and select option matching UPRN
            dropdown = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "ctl00_ContentPlaceHolder1_FF5683DDL")
                )
            )
            # Create a 'Select' for it, then select the matching URPN option
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_value("U" + user_uprn)

            # Wait for the submit button to appear, then click it to get the collection dates
            submit = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "ctl00_ContentPlaceHolder1_btnSubmit")
                )
            )
            submit.click()

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            bins_div = soup.find("div", id="ctl00_ContentPlaceHolder1_FF5686FormGroup")
            if bins_div:
                bins_table = bins_div.find("table")
                if bins_table:
                    # Get table rows
                    for row in bins_table.find_all("tr"):
                        # Get the rows cells
                        cells = row.find_all("td")
                        bin_type = cells[0].get_text(strip=True)
                        # Skip header row
                        if bin_type and cells[3] and bin_type != "Bin Type":
                            if len(cells[3].get_text(strip=True)) > 0:
                                collection_date = datetime.strptime(
                                    cells[3].get_text(strip=True), "%A, %d %B %Y"
                                )
                                dict_data = {
                                    "type": bin_type,
                                    "collectionDate": collection_date.strftime(
                                        date_format
                                    ),
                                }
                                data["bins"].append(dict_data)

                            data["bins"].sort(
                                key=lambda x: datetime.strptime(
                                    x.get("collectionDate"), "%d/%m/%Y"
                                )
                            )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from datetime import datetime

from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = "https://www.salford.gov.uk/bins-and-recycling/bin-collection-days/your-bin-collections"
        user_uprn = kwargs.get("uprn")

        # Check the UPRN is valid
        check_uprn(user_uprn)

        # Create the form data
        params = {
            "UPRN": user_uprn,
        }

        # Make a request to the API
        requests.packages.urllib3.disable_warnings()
        response = requests.get(api_url, params=params)

        # Make a BS4 object
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        # Get the div element
        div_element = soup.find("div", {"class": "wastefurther"})

        # Get the bins
        bin_lists = div_element.find_all("ul")

        # Loop through each <ul> tag to extract the bin information
        for i, bin_list in enumerate(bin_lists):
            # Find the <p> tag containing the bin type string
            bin_type = bin_list.find_previous_sibling("p").find("strong").text.strip()

            # Loop through each <li> tag in the <ul> tag to extract the collection date
            for li in bin_list.find_all("li"):
                # Convert the collection time to a datetime object
                collection_date = datetime.strptime(li.text, "%A %d %B %Y")

                # Add the bin to the data dict
                data["bins"].append(
                    {
                        # remove the ":" from the end of the bin type
                        "type": bin_type[:-1],
                        "collectionDate": collection_date,
                    }
                )

        # Sort the bins by collection time
        data["bins"] = sorted(data["bins"], key=lambda x: x["collectionDate"])

        # Convert the datetime objects to strings in the desired format
        for bin in data["bins"]:
            bin["collectionDate"] = bin["collectionDate"].strftime(date_format)

        return data



from bs4 import BeautifulSoup
import urllib
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        requests.packages.urllib3.disable_warnings()
        root_url = "https://asjwsw-wrpwokingmunicipal-live.whitespacews.com/"
        # Get the house number and postcode from the commandline
        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)

        # Start a new session for the form, and get the chosen URL from the commandline
        session = requests.Session()
        req = session.get(root_url)

        # Parse the requested URL to get a link to the "View My Collections" portal with a unique service ID
        start = BeautifulSoup(req.text, features="html.parser")
        start.prettify()
        base_link = start.select(
            "#menu-content > div > div:nth-child(1) > p.govuk-body.govuk-\!-margin-bottom-0.colorblue.lineheight15 > a"
        )[0].attrs.get("href")

        # We need to reorder the query parts from the unique URL, so split them up to make it easier
        query_parts = urllib.parse.urlparse(base_link).query.split("&")
        parts = base_link.split("?")
        addr_link = (
            parts[0] + "/mop.php?" + query_parts[1] + "&" + query_parts[0] + "&seq=2"
        )

        # Bring in some headers to emulate a browser, and put the UPRN and postcode into the form data.
        # This is sent in a POST request, emulating browser behaviour.
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "Accept-Language": "en-GB,en;q=0.9",
            "Cache-Control": "no-cache",
            "Connection": "keep-alive",
            "Content-Type": "application/x-www-form-urlencoded",
            "Origin": "https://asjwsw-wrpwokingmunicipal-live.whitespacews.com",
            "Pragma": "no-cache",
            "Referer": "https://asjwsw-wrpwokingmunicipal-live.whitespacews.com/",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-User": "?1",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36 OPR/98.0.0.0",
            "sec-ch-ua": '"Chromium";v="112", "Not_A Brand";v="24", "Opera GX";v="98"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
        }
        data = {
            "address_name_number": user_paon,
            "address_street": "",
            "street_town": "",
            "address_postcode": user_postcode,
        }
        addr_page = session.post(addr_link, headers=headers, data=data)
        addr = BeautifulSoup(addr_page.text, features="html.parser")
        addr.prettify()

        # This page should only have one address, but regardless, select the first one and make a request to load the
        # calendar page.
        cal_link = root_url + addr.select("#property_list > ul > li > a")[0].attrs.get(
            "href"
        )
        cal_page = session.get(cal_link)

        # Parse the calendar page
        soup = BeautifulSoup(cal_page.text, features="html.parser")
        soup.prettify()
        data = {"bins": []}

        # For whatever reason, each row contains all the information for that row, and each one after it. This code
        # essentially gets all items from each row, but ignores the whitespace that you get when splitting using \n.
        # This produces a big list of dates then bin types, so we split them up into a list of lists - each pair is
        # a date and the bin type.
        items = [
            i
            for i in soup.find(
                "u1",
                {
                    "class": "displayinlineblock justifycontentleft alignitemscenter margin0 padding0"
                },
            ).text.split("\n")
            if i != ""
        ]
        pairs = [items[i : i + 2] for i in range(0, len(items), 2)]

        # Loop through the paired bin dates and types
        for pair in pairs:
            # This isn't necessary, but better safe than sorry
            collection_date = datetime.strptime(pair[0], date_format).strftime(
                date_format
            )
            # Change the formatting of the purple bins to replace the hyphens with slashes
            if pair[1] == "Batteries-small electricals-textiles":
                bin_type = pair[1].replace("-", "/").strip()
            else:
                bin_type = pair[1]

            # Add the data into the dictionary
            data["bins"].append({"type": bin_type, "collectionDate": collection_date})

        return data
